#! /usr/bin/env python3
# -*- coding: utf-8 -*-
'''
This is where your module level help string goes.

.. module:: `Img`
   :platform: Unix, Windows
   :synopsis: Put a synopsis of what this module does here.

.. moduleauthor:: Tumurtogtokh Davaakhuu <tumurtogtokh@gmail.com>
'''
# IMPORT STANDARD
import time
import os
import logging
from urllib.parse import urlparse
from urllib.request import urlretrieve
import threading

# IMPORT Local

logging.basicConfig(filename='logfile.log', level=logging.DEBUG)


class Img:
    '''
    Image Wrapper
    '''

    def __init__(self, home_dir='./data/', max_con_dl=0):
        self.home_dir = home_dir
        self.input_dir = self.home_dir + os.path.sep + 'incoming'
        self.output_dir = self.home_dir + os.path.sep + 'outgoing'
        self.downloaded_bytes = 0
        self.download_lock = threading.Lock()
        if max_con_dl > 1:
            self.max_concurrent_dl = max_con_dl
            self.sem_lock = threading.Semaphore(max_con_dl)
        else:
            self.sem_lock = threading.Semaphore()
        os.makedirs(self.input_dir, exist_ok=True)
        os.makedirs(self.output_dir, exist_ok=True)

    def download_img(self, url, keep_log=False):
        # download each image and save to the input dir
        img_filename = urlparse(url).path.split('/')[-1]
        dest_path = self.input_dir + os.path.sep + img_filename
        
        # downloading images with limit of self.max_concurrent_dl
        self.sem_lock.acquire()
        try:
            with self.download_lock:
                urlretrieve(url, dest_path)
                img_bytes = os.path.getsize(dest_path)
                self.downloaded_bytes += img_bytes
            if keep_log:
                logging.info("<Img> Image downloaded to: " + dest_path)
                logging.info("<Img> Image size: " + str(img_bytes) + ' bytes')
        finally:
            self.sem_lock.release()    
    
    def download_images(self, img_url_list, keep_log=True):
        '''
        Download images from url using threads
        '''
        # validation
        if not img_url_list:
            return
        os.makedirs(self.input_dir, exist_ok=True)
        # ---------------------------------------------------------------------

        # start time for logging
        start = time.perf_counter()

        threads = []
        # Download starts
        for url in img_url_list:
            t = threading.Thread(target=self.download_img, args=(url,))
            t.start()
            threads.append(t)

        for thread in threads:
            thread.join()

        # end time for loggin
        end = time.perf_counter()

        if keep_log:
            logging.info("<Img> Downloaded: {} images in {} seconds".
                         format(len(img_url_list), end - start))


# -----------------------------------------------------------------------------
__all__ = ['Img']


def main():
    '''
    Just runs help if necessary
    '''
    import __main__
    help(__main__)


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
'''
This is where your module level help string goes.

.. module:: `Main`
   :platform: Unix, Windows
   :synopsis: Put a synopsis of what this module does here.

.. moduleauthor:: Tumurtogtokh Davaakhuu <tumurtogtokh@gmail.com>

------------------------------------------------------------
USE: python <PROGNAME> (options)
OPTIONS:
    -h : print this help message
    -t FILE : target size FILE
    -i FILE : image urls FILE
    -l : keep log configuration (default: without)
------------------------------------------------------------
'''
# IMPORT STANDARD
import sys
import os

# IMPORT Local
from imgresizer import Img
from imgresizer import ImageSizerController
from imgresizer import CommandLine

# =============================================================================
# MAIN


def main():
    config = CommandLine()

    if config.exit:
        sys.exit(0)
    IMG_URLS = config.process_img_url_file()
    TARGET = config.process_target_file()
    HOME = os.getcwd()
    DATA = 'data'

    img_sizer = ImageSizerController(Img(DATA), IMG_URLS, TARGET)
    # img_sizer.perform_resizing()
    img_sizer.make_imgs()


if __name__ == '__main__':
    main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
'''
This is where your module level help string goes.

.. module:: `Image Re-Sizer`
   :platform: Unix, Windows
   :synopsis: Put a synopsis of what this module does here.

.. moduleauthor:: Tumurtogtokh Davaakhuu <tumurtogtokh@gmail.com>
'''
# IMPORT STANDARD
import time
import os
import logging
from queue import Queue
from threading import Thread
from urllib.parse import urlparse
from urllib.request import urlretrieve

# IMPORT Local
import PIL
from PIL import Image

FORMAT = "[%(threadName)s, %(asctime)s, %(levelname)s, %(message)s]"
logging.basicConfig(filename='logfile.log', level=logging.DEBUG, format=FORMAT)


class ImageSizerController:
    '''
    ImageSizer Processor
    '''

    def __init__(self, img, img_url_list, targets, log=True, num_threas=16):
        '''
        :arg img: Img class
        :type img: Img object
        :arg img_url_list: list of img urls
        :type img_url_list: str list
        :arg targets: image height sizes
        :type targets: int list
        '''
        self.images = img
        # self.images.download_images_queue(img_url_list, log)
        self.target_sizes = targets
        self.keep_log = log
        self.img_queue = Queue()
        self.dl_queue = Queue()
        self.img_list = img_url_list
        self.num_threads = 16

    def download_imgs(self, keep_log=True):
        # start time for logging
        start = time.perf_counter()
        while not self.dl_queue.empty():
            try:
                url = self.dl_queue.get(block=False)
                img_filename = urlparse(url).path.split('/')[-1]
                dest_path = self.images.input_dir + os.path.sep + img_filename
                urlretrieve(url, dest_path)
                self.img_queue.put(img_filename)

                self.dl_queue.task_done()
            except Queue.Empty as err:
                logging.info(err)

        # end time for logging
        end = time.perf_counter()
        if keep_log:
            logging.info("Downloaded: {} images in {} seconds".
                         format(len(self.img_list), end - start))

    def perform_resizing(self):
        '''
        Processing images
        '''
        os.makedirs(self.images.output_dir, exist_ok=True)
        # ---------------------------------------------------------------------
        num_images = len(os.listdir(self.images.input_dir))
        # start time for logging
        start = time.perf_counter()

        # Processing
        while True:
            filename = self.img_queue.get()
            if filename:
                orig_img = Image.open(
                    self.images.input_dir + os.path.sep + filename)
                for basewidth in self.target_sizes:
                    tmp_img = orig_img
                    # calculate target height of the resized image
                    # to maintain the aspect ratio
                    wpercent = (basewidth / float(tmp_img.size[0]))
                    hsize = int((float(tmp_img.size[1]) * float(wpercent)))
                    # perform resizing
                    tmp_img = tmp_img.resize(
                        (basewidth, hsize), PIL.Image.LANCZOS)
                    # save the resized image to the output dir with a modified name
                    new_filename = os.path.splitext(filename)[0] + \
                        '_' + str(basewidth) + os.path.splitext(filename)[1]
                    tmp_img.save(self.images.output_dir +
                                 os.path.sep + new_filename)

                # os.remove(self.images.input_dir + os.path.sep + filename)
                self.img_queue.task_done()
            else:
                self.img_queue.task_done()
                break
        # end time for logging
        end = time.perf_counter()

        if self.keep_log:
            logging.info("Created {} thumbnails in {} seconds".
                         format(num_images, end - start))

    def make_imgs(self):
        start = time.perf_counter()
        logging.info('Making images')

        for img_url in self.img_list:
            self.dl_queue.put(img_url)
        # starting downloading with threads
        for _ in range(self.num_threads):
            tr = Thread(target=self.download_imgs)
            tr.start()

        resize_thread = Thread(target=self.perform_resizing)
        resize_thread.start()

        self.dl_queue.join()
        self.img_queue.put(None)

        resize_thread.join()
        end = time.perf_counter()
        logging.info("Whole Process took: {}".format(end - start))


__all__ = ['ImageSizerController']


def main():
    '''
    Just runs help if necessary
    '''
    import __main__
    help(__main__)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = ["requests>=2", "Pillow>=5.4", "imagesize>=1.1"]

setup(
    name="imgresizer",
    version="0.0.1",
    author="Tumurtogtokh Davaakhuu",
    author_email="tumurtogtokh@gmail.com",
    description="A package to re-size images",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/UnboxingMinds/ImgReSizer",
    packages=find_packages(),
    install_requires = requirements,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    include_package_data = True,
    entry_points={
        "console_scripts":[
            "imgresizer=imgresizer.__main__:main",
        ]
    },
)


#! /usr/bin/env python3
# -*- coding: utf-8 -*-
'''
------------------------------------------------------------
USE: python <PROGNAME> (options)
OPTIONS:
    -h : print this help message
    -t FILE : target size FILE
    -i FILE : image urls FILE
    -l : keep log configuration (default: without)
------------------------------------------------------------
'''
# IMPORT STANDARD
import getopt
import sys
import os

# GLOBAL CONSTANTS
TARGET_ERR = "** ERROR: must specify target sizes file (opt: -t FILE) **"
IMG_URL_ERR = "** ERROR: must specify image url file (opt: -i FILE) **"
ARG_ERR = "** ERROR: must specify correct arguments **kwargs **"

# =============================================================================
# Command line processing

class CommandLine:
    '''
    CLI interface for controlling the app
    '''
    def __init__(self):
        self.exit = False
        self.img_urls_file = ''
        self.target_file = ''
        self.keep_log = False

        try:
            opts, args = getopt.getopt(sys.argv[1:], 'hl:t:i:l',
                                       ['help', 'target sizes', 'img url', 'keeplog'])
            opts = dict(opts)

        # ---------------------------------------------------------------------
            if '-h' in opts:
                self.print_help()
                self.exit = True
                return
            
            if not ('-t' in opts and '-i' in opts):
                print(ARG_ERR, file=sys.stderr)
                self.print_help()
                return

            if '-t' in opts:
                self.target_file = opts['-t']
            
            if '-l' in opts:
                if str(opts['-l']).lower() == 'false':
                    self.keep_log = False
                else:
                    self.keep_log = True
            
            if '-i' in opts:
                self.img_urls_file = opts['-i']

        except getopt.GetoptError as err:
            self.exit = True
            print(ARG_ERR)
            self.print_help()

    def print_help(self):
        '''
        Displays help
        '''
        prog_name = sys.argv[0]
        print(__doc__.replace('<PROGNAME>', prog_name, 1), file=sys.stderr)

    def process_img_url_file(self):
        '''
        Return list of url
        '''
        urls = []
        if not os.path.isfile(self.img_urls_file):
            print(IMG_URL_ERR, file=sys.stderr)
            return urls
        with open(self.img_urls_file, 'r') as fin:
            for line in fin:
                if line.strip():
                    urls.append(line.strip())
        return urls

    def process_target_file(self):
        '''
        Returns target size list from input arg
        '''
        target = []
        if not os.path.isfile(self.target_file):
            print(TARGET_ERR, file=sys.stderr)
            return target
        with open(self.target_file, 'r') as fin:
            for line in fin:
                if line.strip():
                    target.append(int(line.strip()))
        return target

if __name__ == '__main__':
    cli = CommandLine()
    cli.exit = True
